@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2013-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2013-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal  Not intended for use outside of the ActiveSync library.
@@ -77,11 +77,19 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
 
     /**
      * Global WINDOWSIZE
+     * Defaults to 100 (MS-ASCMD 2.2.3.188)
      *
      * @var integer
      */
     protected $_globalWindowSize = 100;
 
+    /**
+     * Flag to indicate we have overridden the globalWindowSize
+     *
+     * @var boolean
+     */
+    protected $_windowsizeOverride = false;
+
     /**
      * Imported changes flag.
      *
@@ -334,7 +342,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      * @param $id  The uid.
      *
      * @return string  The backend server id.
-     * @throws Horde_ActiveSync_Exception
+     * @throws Horde_ActiveSync_Exception_FolderGone
      */
     public function getBackendIdForFolderUid($folderid)
     {
@@ -349,7 +357,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
             $this->_logger->err(
                 sprintf('[%s] Horde_ActiveSync_Collections::getBackendIdForFolderUid failed because folder was not found in cache.',
                 $this->_procid));
-            throw new Horde_ActiveSync_Exception('Folder not found in cache.');
+            throw new Horde_ActiveSync_Exception_FolderGone('Folder not found in cache.');
         }
     }
 
@@ -432,11 +440,19 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      * that can be sent (including all collections). This method should be
      * renamed for 3.0
      *
-     * @param integer $window  The windowsize
+     * @param integer $window    The windowsize
+     * @param boolean $override  If true, this value will override any client
+     *                           supplied value.
      */
-    public function setDefaultWindowSize($window)
+    public function setDefaultWindowSize($window, $override = false)
     {
-        $this->_globalWindowSize = $window;
+        if ($override) {
+            $this->_windowsizeOverride = true;
+        }
+
+        if ($override || empty($this->_windowsizeOverride)) {
+            $this->_globalWindowSize = $window;
+        }
     }
 
     public function getDefaultWindowSize()
@@ -484,6 +500,13 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         if ($id == 'RI') {
             return $id;
         }
+
+        // First try existing, loaded collections.
+        if (!empty($this->_collections[$id])) {
+            return $this->_collections[$id]['class'];
+        }
+
+        // Next look in the SyncCache.
         if (isset($this->_cache->folders[$id]['class'])) {
             $class = $this->_cache->folders[$id]['class'];
             $this->_logger->info(sprintf(
@@ -495,6 +518,29 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         return false;
     }
 
+    public function getCollectionType($id)
+    {
+        if ($id == 'RI') {
+            return $id;
+        }
+
+        // First try existing, loaded collections.
+        if (!empty($this->_collections[$collection['id']])) {
+            return $this->_collections[$collection['id']]['type'];
+        }
+
+        // Next look in the SyncCache.
+        if (isset($this->_cache->folders[$id]['type'])) {
+            $type = $this->_cache->folders[$id]['type'];
+            $this->_logger->info(sprintf(
+                '[%s] Obtaining collection type of %s for collection id %s',
+                $this->_procid, $type, $id));
+            return $type;
+        }
+
+        return false;
+    }
+
     /**
      * Determine if we have any syncable collections either locally or in the
      * sync cache.
@@ -1010,26 +1056,31 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      *                                 throw exception if it's not present.
      *
      * @throws Horde_ActiveSync_Exception_InvalidRequest
+     * @throws Horde_ActiveSync_Exception_FolderGone
      */
-    public function initCollectionState(array $collection, $requireSyncKey = false)
+    public function initCollectionState(array &$collection, $requireSyncKey = false)
     {
         // Clear the changes cache.
         $this->_changes = null;
 
+        // Ensure we have a collection class.
         if (empty($collection['class'])) {
-            if (!empty($this->_collections[$collection['id']])) {
-                $collection['class'] = $this->_collections[$collection['id']]['class'];
-            } else {
+            if (!($collection['class'] = $this->getCollectionClass($collection['id']))) {
                 throw new Horde_ActiveSync_Exception_FolderGone('Could not load collection class for ' . $collection['id']);
             }
         }
 
+        // Load the collection's type if we can.
+        if (empty($collection['type'])) {
+            $collection['type'] = $this->getCollectionType($collection['id']);
+        }
+
         // Get the backend serverid.
         if (empty($collection['serverid'])) {
-            $f = $this->_cache->getFolder($collection['id']);
-            $collection['serverid'] = $f['serverid'];
+            $collection['serverid'] = $this->getBackendIdForFolderUid($collection['id']);
         }
 
+
         if ($requireSyncKey && empty($collection['synckey'])) {
             throw new Horde_ActiveSync_Exception_InvalidRequest(sprintf(
                 '[%s] Empty synckey for %s.',
@@ -1086,7 +1137,11 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         $this->lasthbsyncstarted = $started;
         $this->save();
 
+        // We only check for remote wipe request once every 5 iterations to
+        // save on DB load since we must reload the device's state each time.
+        $rw_check_countdown = 5;
         while (($now = time()) < $until) {
+
             // Try not to go over the heartbeat interval.
             if ($until - $now < $interval) {
                 $interval = $until - $now;
@@ -1105,12 +1160,15 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                 return self::COLLECTION_ERR_SERVER;
             }
 
-            // Check for WIPE request. If so, force a foldersync so it is
-            // performed.
-            if ($this->_as->provisioning != Horde_ActiveSync::PROVISIONING_NONE) {
-                $rwstatus = $this->_as->state->getDeviceRWStatus($this->_as->device->id);
-                if ($rwstatus == Horde_ActiveSync::RWSTATUS_PENDING || $rwstatus == Horde_ActiveSync::RWSTATUS_WIPED) {
-                    return self::COLLECTION_ERR_FOLDERSYNC_REQUIRED;
+            // Check for WIPE request once every 5 iterations to balance between
+            // performance and speed of catching a remote wipe request.
+            if ($rw_check_countdown-- == 0) {
+                 $rw_check_countdown = 5;
+                 if ($this->_as->provisioning != Horde_ActiveSync::PROVISIONING_NONE) {
+                    $rwstatus = $this->_as->state->getDeviceRWStatus($this->_as->device->id, true);
+                    if ($rwstatus == Horde_ActiveSync::RWSTATUS_PENDING || $rwstatus == Horde_ActiveSync::RWSTATUS_WIPED) {
+                        return self::COLLECTION_ERR_FOLDERSYNC_REQUIRED;
+                    }
                 }
             }
 
