@@ -5,7 +5,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -19,7 +19,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -190,7 +190,11 @@ class Horde_ActiveSync_Imap_Adapter
         // the client of the error.
         $search_q = new Horde_Imap_Client_Search_Query();
         $search_q->ids($ids_obj);
-        $fetch_res = $imap->search($mbox, $search_q);
+        try {
+            $fetch_res = $imap->search($mbox, $search_q);
+        } catch (Horde_Imap_Client_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
         if ($fetch_res['count'] != count($uids)) {
             $ids_obj = $fetch_res['match'];
         }
@@ -266,6 +270,8 @@ class Horde_ActiveSync_Imap_Adapter
      *                DEFAULT: false (Do not fetch headers).
      *
      * @return array  An array of Horde_ActiveSync_Imap_Message objects.
+     * @todo This should be renamed to getImapMessages since we can now accept
+     *       an array of $uids.
      */
     public function getImapMessage($mailbox, $uid, array $options = array())
     {
@@ -1508,6 +1514,7 @@ class Horde_ActiveSync_Imap_Adapter
     protected function _doQuery(array $query)
     {
         $imap_query = new Horde_Imap_Client_Search_Query();
+        $imap_query->charset('UTF-8', false);
         $mboxes = array();
         $results = array();
         foreach ($query as $q) {
