@@ -5,7 +5,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -21,7 +21,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -104,11 +104,14 @@ class Horde_ActiveSync_Imap_Message
     {
         switch ($property) {
         case 'envelope':
-            return $this->_data->getEnvelope();
+            $e = $this->_data->getEnvelope();
+            return $e;
         case 'flags':
-            return $this->_data->getFlags();
+            $f = $this->_data->getFlags();
+            return $f;
         case 'uid':
-            return $this->_data->getUid();
+            $u = $this->_data->getUid();
+            return $u;
         case 'basePart':
             if (empty($this->_basePart)) {
                 $this->_basePart = new Horde_ActiveSync_Mime($this->_data->getStructure());
@@ -204,7 +207,7 @@ class Horde_ActiveSync_Imap_Message
                     $query,
                     array('ids' => new Horde_Imap_Client_Ids(array($this->uid)))
                 );
-            } catch (Horde_Imap_Exception $e) {
+            } catch (Horde_Imap_Client_Exception $e) {
                 throw new Horde_ActiveSync_Exception($e);
             }
             $data = $fetch_ret[$this->uid];
@@ -288,8 +291,10 @@ class Horde_ActiveSync_Imap_Message
     public function getAttachments($version)
     {
         $ret = array();
-        $map = $this->basePart->contentTypeMap();
-        foreach ($map as $id => $type) {
+        $iterator = new Horde_ActiveSync_Mime_Iterator($this->_basePart->base);
+        foreach ($iterator as $part) {
+            $type = $part->getType();
+            $id = $part->getMimeId();
             if ($this->isAttachment($id, $type)) {
                 if ($type != 'application/ms-tnef' || (!$mime_part = $this->_decodeTnefData($id))) {
                     $mime_part = $this->getMimePart($id, array('nocontents' => true));
@@ -396,15 +401,6 @@ class Horde_ActiveSync_Imap_Message
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
                 $mpart = $this->getMimePart($id);
-                // Work around PHP bug 65776
-                if ($mpart->getPrimaryType() == 'text') {
-                    $mpart->setContents(
-                        $mpart->replaceEOL(
-                            $mpart->getContents(array('stream' => true)), Horde_Mime_Part::RFC_EOL, true
-                        ),
-                        array('usestream' => true)
-                    );
-                }
                 if ($mpart->getType() == 'text/calendar') {
                     $mpart->setDisposition('inline');
                 }
