@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -31,7 +31,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -73,6 +73,15 @@ class Horde_ActiveSync_Message_Base
      */
     public $flags = false;
 
+    /**
+     * Request type. One of: Horde_ActiveSync::SYNC_ADD, SYNC_MODIFY,
+     *    SYNC_REMOVE, or SYNC_FETCH. Used internally for enforcing various
+     *    protocol rules depending on request. @since 2.31.0
+     *
+     * @var  string
+     */
+    public $commandType;
+
     /**
      * Logger
      *
@@ -88,7 +97,7 @@ class Horde_ActiveSync_Message_Base
     protected $_supported = array();
 
     /**
-     * Existance cache, used for working with ghosted properties.
+     * Existence cache, used for working with ghosted properties.
      *
      * @var array
      */
@@ -109,6 +118,11 @@ class Horde_ActiveSync_Message_Base
      */
     protected $_device;
 
+    /**
+     * Cache of current stream filters.
+     *
+     * @var array
+     */
     protected $_streamFilters = array();
 
     /**
@@ -203,6 +217,30 @@ class Horde_ActiveSync_Message_Base
         $this->_exists[$property] = true;
     }
 
+    /**
+     * Give concrete classes the chance to enforce rules.
+     *
+     * @return boolean  True on success, otherwise false.
+     * @since  2.31.0
+     */
+    protected function _validateDecodedValues()
+    {
+        return true;
+    }
+
+    /**
+     * Give concrete classes the chance to enforce rules before encoding
+     * messages to send to the client.
+     *
+     * @return boolean  True if values were valid (or could be made valid).
+     *     False if values are unable to be validated.
+     * @since  2.31.0
+     */
+    protected function _preEncodeValidation()
+    {
+        return true;
+    }
+
     /**
      * Magic caller method.
      *
@@ -239,11 +277,20 @@ class Horde_ActiveSync_Message_Base
     /**
      * Set the list of non-ghosted fields for this message.
      *
-     * @param array $fields  The array of fields.
+     * @param array $fields  The array of fields, keyed by the fully qualified
+     *                       property name i.e., POOMCONTACTS:Anniversary. To
+     *                       signify an empty SUPPORTED container $fields should
+     *                       contain a single element equal to
+     *                       Horde_ActiveSync::ALL_GHOSTED.
      */
     public function setSupported(array $fields)
     {
         $this->_supported = array();
+        if (current($fields) == Horde_ActiveSync::ALL_GHOSTED) {
+            $this->_supported = $fields;
+            return;
+        }
+
         foreach ($fields as $field) {
             $this->_supported[] = $this->_mapping[$field][self::KEY_ATTRIBUTE];
         }
@@ -261,11 +308,8 @@ class Horde_ActiveSync_Message_Base
 
     /**
      * Determines if the property specified has been ghosted by the client.
-     * A ghosted property 1) IS listed in the supported list and 2) NOT
-     * present in the current message. If it's IN the supported list and NOT
-     * in the current message, then it IS ghosted and the server should keep
-     * the field's current value when performing any change action due to this
-     * message.
+     * A property is ghosted if it is NOT listed in the SUPPORTED list sent
+     * by the client AND is NOT present in the request data.
      *
      * @param string $property  The property to check
      *
@@ -273,13 +317,21 @@ class Horde_ActiveSync_Message_Base
      */
     public function isGhosted($property)
     {
-        if (array_search($property, $this->_supported) === false) {
+        // MS-ASCMD 2.2.3.168:
+        // An empty SUPPORTED container indicates that ALL elements able to be
+        // ghosted ARE ghosted. A *missing* SUPPORTED tag indicates that NO
+        // fields are ghosted - any ghostable properties are always considered
+        // NOT ghosted.
+        if (empty($this->_supported)) {
             return false;
-        } elseif (empty($this->_exists[$property])) {
+        }
+        if (current($this->_supported) == Horde_ActiveSync::ALL_GHOSTED &&
+            empty($this->_exists[$property])) {
             return true;
         }
 
-        return false;
+        return array_search($property, $this->_supported) === false &&
+               empty($this->_exists[$property]);
     }
 
     /**
@@ -310,18 +362,21 @@ class Horde_ActiveSync_Message_Base
 
                 // Found start tag
                 if (!isset($this->_mapping[$entity[Horde_ActiveSync_Wbxml::EN_TAG]])) {
-                    $this->_logger->err('Tag ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG] . ' unexpected in type XML type ' . get_class($this));
+                    $this->_logger->err(sprintf(
+                        'Tag %s unexpected in type XML type %s.',
+                         $entity[Horde_ActiveSync_Wbxml::EN_TAG],
+                         get_class($this))
+                    );
                     throw new Horde_ActiveSync_Exception('Unexpected tag');
                 } else {
                     $map = $this->_mapping[$entity[Horde_ActiveSync_Wbxml::EN_TAG]];
                     if (isset($map[self::KEY_VALUES])) {
                         // Handle arrays of attribute values
                         while (1) {
-                            //do not get start tag for an array without a container
-                            if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER)) {
-                                if (!$decoder->getElementStartTag($map[self::KEY_VALUES])) {
-                                    break;
-                                }
+                            // Do not get start tag for an array without a container
+                            if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER) &&
+                                !$decoder->getElementStartTag($map[self::KEY_VALUES])) {
+                                break;
                             }
                             if (isset($map[self::KEY_TYPE])) {
                                 $class = $map[self::KEY_TYPE];
@@ -329,6 +384,7 @@ class Horde_ActiveSync_Message_Base
                                     'protocolversion' => $this->_version,
                                     'logger' => $this->_logger)
                                 );
+                                $decoded->commandType = $this->commandType;
                                 $decoded->decodeStream($decoder);
                             } else {
                                 $decoded = $decoder->getElementContent();
@@ -338,28 +394,28 @@ class Horde_ActiveSync_Message_Base
                             } else {
                                 $this->{$map[self::KEY_ATTRIBUTE]}[] = $decoded;
                             }
-
                             if (!$decoder->getElementEndTag()) {
                                 throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
                             }
                             if (isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER) {
                                 $e = $decoder->peek();
-                                //go back to the initial while if another block of no container elements is found
+                                // Go back to the initial while if another block
+                                // of a non-container element is found.
                                 if ($e[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_STARTTAG) {
                                     continue 2;
                                 }
-                                //break on end tag because no container elements block end is reached
-                                if ($e[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_ENDTAG)
-                                    break;
-                                if (empty($e))
+                                // Break on end tag because no other container
+                                // elements block end is reached.
+                                if ($e[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_ENDTAG || empty($e)) {
                                     break;
+                                }
                             }
                         }
-                        //do not get container end tag for an array without a container
-                        if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER)) {
-                            if (!$decoder->getElementEndTag()) {
-                                return false;
-                            }
+
+                        // Do not get container end tag for an array without a container
+                        if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER) &&
+                            !$decoder->getElementEndTag()) {
+                            return false;
                         }
                     } else {
                         // Handle a simple attribute value
@@ -375,6 +431,7 @@ class Horde_ActiveSync_Message_Base
                                     'protocolversion' => $this->_version,
                                     'logger' => $this->_logger)
                                 );
+                                $subdecoder->commandType = $this->commandType;
                                 $subdecoder->decodeStream($decoder);
                                 $decoded = $subdecoder;
                             }
@@ -383,11 +440,17 @@ class Horde_ActiveSync_Message_Base
                             $decoded = $decoder->getElementContent();
                             if ($decoded === false) {
                                 $decoded = '';
-                                $this->_logger->notice('Unable to get expected content for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG] . ': Setting to an empty string.');
+                                $this->_logger->notice(sprintf(
+                                    'Unable to get expected content for %s: Setting to an empty string.',
+                                    $entity[Horde_ActiveSync_Wbxml::EN_TAG])
+                                );
                             }
                         }
                         if (!$decoder->getElementEndTag()) {
-                            $this->_logger->err('Unable to get end tag for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG]);
+                            $this->_logger->err(sprintf(
+                                'Unable to get end tag for %s.',
+                                $entity[Horde_ActiveSync_Wbxml::EN_TAG])
+                            );
                             throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
                         }
                         $this->{$map[self::KEY_ATTRIBUTE]} = $decoded;
@@ -401,6 +464,12 @@ class Horde_ActiveSync_Message_Base
                 break;
             }
         }
+        if (!$this->_validateDecodedValues()) {
+            throw new Horde_ActiveSync_Exception(sprintf(
+                'Invalid values detected in %s.',
+                get_class($this))
+            );
+        }
     }
 
     /**
@@ -411,12 +480,19 @@ class Horde_ActiveSync_Message_Base
      */
     public function encodeStream(Horde_ActiveSync_Wbxml_Encoder &$encoder)
     {
+        if (!$this->_preEncodeValidation()) {
+            throw new Horde_ActiveSync_Exception(sprintf(
+                'Pre-encoding validation failded for %s item',
+                get_class($this))
+            );
+        }
+
         foreach ($this->_mapping as $tag => $map) {
             if (isset($this->{$map[self::KEY_ATTRIBUTE]})) {
                 // Variable is available
                 if (is_object($this->{$map[self::KEY_ATTRIBUTE]}) &&
                     !($this->{$map[self::KEY_ATTRIBUTE]} instanceof Horde_Date)) {
-                    // Subobjects can do their own encoding
+                    // Objects can do their own encoding
                     $encoder->startTag($tag);
                     $this->{$map[self::KEY_ATTRIBUTE]}->encodeStream($encoder);
                     $encoder->endTag();
@@ -597,7 +673,7 @@ class Horde_ActiveSync_Message_Base
     {
         $len = strlen($data);
         $newdata = '';
-        for($i = 0;$i < $len;$i += 2)
+        for($i = 0; $i < $len; $i += 2)
         {
             $newdata .= pack('C', hexdec(substr($data, $i, 2)));
         }
