@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,7 +30,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal
@@ -274,6 +274,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
         $pingSettings = $this->_driver->getHeartbeatConfig();
 
+        // Override the total, per-request, WINDOWSIZE?
+        if (!empty($pingSettings['maximumrequestwindowsize'])) {
+            $this->_collections->setDefaultWindowSize($pingSettings['maximumrequestwindowsize'], true);
+        }
+
         // If this is >= 12.1, see if we want a looping SYNC.
         if ($this->_collections->canDoLoopingSync() &&
             $this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE &&
@@ -490,13 +495,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         ? min($collection['windowsize'], $pingSettings['maximumwindowsize'])
                         : $collection['windowsize'];
 
-                    if ((!empty($changecount) && $changecount > $max_windowsize) ||
-                        ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
+                    if (!empty($changecount) &&
+                        (($changecount > $max_windowsize) || $cnt_global + $max_windowsize > $this->_collections->getDefaultWindowSize())) {
                         $this->_logger->info(sprintf(
-                            '[%s] Sending MOREAVAILABLE. WINDOWSIZE = %d, $changecount = %d, $cnt_global = %d',
-                            $this->_procid, $max_windowsize, $changecount, $cnt_global));
+                            '[%s] Sending MOREAVAILABLE. WINDOWSIZE = %d, $changecount = %d, MAX_REQUEST_WINDOWSIZE = %d, $cnt_global = %d',
+                            $this->_procid, $max_windowsize, $changecount, $this->_collections->getDefaultWindowSize(), $cnt_global));
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
-                        $over_window = ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize());
+                        $over_window = ($cnt_global + $max_windowsize > $this->_collections->getDefaultWindowSize());
                     }
 
                     if (!empty($changecount)) {
@@ -800,6 +805,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     }
                     while (1) {
                         $el = $this->_decoder->getElement();
+                        if ($this->_decoder->isEmptyElement($this->_decoder->getLastStartElement())) {
+                            // MS-ASCMD 2.2.3.168 An empty SUPPORTED tag
+                            // indicates that ALL elements able to be ghosted
+                            // ARE ghosted.
+                            $collection['supported'] = array(Horde_ActiveSync::ALL_GHOSTED);
+                            break;
+                        }
                         if ($el[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_ENDTAG) {
                             break;
                         }
@@ -916,13 +928,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_procid));
         }
 
-        if (empty($collection['class'])) {
-            $collection['class'] = $this->_collections->getCollectionClass($collection['id']);
-        }
-        if (empty($collection['serverid'])) {
-            $collection['serverid'] = $this->_collections->getBackendIdForFolderUid($collection['id']);
-        }
-
         try {
             $this->_collections->initCollectionState($collection);
         } catch (Horde_ActiveSync_Exception_StateGone $e) {
@@ -937,6 +942,10 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             $this->_statusCode = self::STATUS_SERVERERROR;
             $this->_handleGlobalSyncError();
             return false;
+        } catch (Horde_ActiveSync_Exception $e) {
+            $this->_statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
+            $this->_handleError($colleciton);
+            return false;
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
             $this->_statusCode = self::STATUS_SERVERERROR;
@@ -958,11 +967,12 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 break;
             }
             $nchanges++;
-
+            $commandType = $element[Horde_ActiveSync_Wbxml::EN_TAG];
+            $instanceid = false;
             // Only sent during SYNC_MODIFY/SYNC_REMOVE/SYNC_FETCH
-            if (($element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_MODIFY ||
-                 $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_REMOVE ||
-                 $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_FETCH) &&
+            if (($commandType == Horde_ActiveSync::SYNC_MODIFY ||
+                 $commandType == Horde_ActiveSync::SYNC_REMOVE ||
+                 $commandType == Horde_ActiveSync::SYNC_FETCH) &&
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_SERVERENTRYID)) {
 
                 $serverid = $this->_decoder->getElementContent();
@@ -974,6 +984,18 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_logger->err('Parsing Error - expecting </SYNC_SERVERENTRYID>');
                     return false;
                 }
+
+                if ($this->_activeSync->device->version >= Horde_ActiveSync::VERSION_SIXTEEN) {
+                    if ($this->_decoder->getElementStartTag(Horde_ActiveSync::AIRSYNCBASE_INSTANCEID)) {
+                        $instanceid = $this->_decoder->getElementContent();
+                        if ($instanceid !== false && !$this->_decoder->getElementEndTag()) {
+                            $this->_statusCode = self::STATUS_PROTERROR;
+                            $this->_handleGlobalSyncError();
+                            $this->_logger->err('Parsing Error - expecting </AIRSYNCBASE_INSTANCEID>');
+                            return false;
+                        }
+                    }
+                }
             } else {
                 $serverid = false;
             }
@@ -982,7 +1004,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             // and it's not even sent by all clients. Parse it if it's there,
             // ignore it if not.
             if ($this->_activeSync->device->version > Horde_ActiveSync::VERSION_TWELVEONE &&
-                $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_ADD &&
+                $commandType == Horde_ActiveSync::SYNC_ADD &&
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_FOLDERTYPE)) {
 
                 $collection['class'] = $this->_decoder->getElementContent();
@@ -995,7 +1017,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             }
 
             // Only sent during SYNC_ADD
-            if ($element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_ADD &&
+            if ($commandType == Horde_ActiveSync::SYNC_ADD &&
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_CLIENTENTRYID)) {
                 $clientid = $this->_decoder->getElementContent();
                 if (!$this->_decoder->getElementEndTag()) {
@@ -1010,8 +1032,8 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
             // Create Message object from messages passed from client.
             // Only passed during SYNC_ADD or SYNC_MODIFY
-            if (($element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_ADD ||
-                $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_MODIFY) &&
+            if (($commandType == Horde_ActiveSync::SYNC_ADD ||
+                $commandType == Horde_ActiveSync::SYNC_MODIFY) &&
                 $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_DATA)) {
                 switch ($collection['class']) {
                 case Horde_ActiveSync::CLASS_EMAIL:
@@ -1025,6 +1047,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 case Horde_ActiveSync::CLASS_CALENDAR:
                     $appdata = Horde_ActiveSync::messageFactory('Appointment');
                     $appdata->decodeStream($this->_decoder);
+                    if (!empty($instanceid) &&
+                        $commandType == Horde_ActiveSync::SYNC_MODIFY) {
+                        // EAS 16.0 sends instanceid/serverid for exceptions.
+                        $appdata->instanceid = $instanceid;
+                    }
                     break;
                 case Horde_ActiveSync::CLASS_TASKS:
                     $appdata = Horde_ActiveSync::messageFactory('Task');
@@ -1047,13 +1074,16 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     return false;
                 }
             }
+            $appdata->commandType = $commandType;
 
             if (!empty($collection['synckey'])) {
-                switch ($element[Horde_ActiveSync_Wbxml::EN_TAG]) {
+                switch ($commandType) {
                 case Horde_ActiveSync::SYNC_MODIFY:
                     if (isset($appdata)) {
                         $id = $importer->importMessageChange(
-                            $serverid, $appdata, $this->_device, false);
+                            $serverid, $appdata, $this->_device, false,
+                            $collection['class'], $collection['synckey']
+                        );
                         if ($id && !is_array($id)) {
                             $collection['importedchanges'] = true;
                         } elseif (is_array($id)) {
@@ -1076,8 +1106,10 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     break;
 
                 case Horde_ActiveSync::SYNC_REMOVE:
-                    // Work around broken clients that send empty $serverid.
-                    if ($serverid) {
+                    if ($instanceid) {
+                        $collection['instanceid_removes'][$serverid] = $instanceid;
+                    } elseif ($serverid) {
+                        // Work around broken clients that send empty $serverid.
                         $collection['removes'][] = $serverid;
                     }
                     break;
@@ -1114,6 +1146,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             unset($collection['removes']);
             $collection['importedchanges'] = true;
         }
+        // EAS 16.0 instance deletions.
+        if (!empty($collection['instanceid_removes']) && !empty($collection['synckey'])) {
+                foreach ($collection['instanceid_removes'] as $uid => $instanceid) {
+                    $importer->importMessageDeletion(array($uid => $instanceid), $collection['class'], true);
+                }
+            unset($collection['instanceid_removes']);
+        }
 
         $this->_logger->info(sprintf(
             '[%s] Processed %d incoming changes',
