@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -65,7 +65,7 @@
  *          - pingheartbeat:
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- * @copyright 2010-2015 Horde LLC (http://www.horde.org/)
+ * @copyright 2010-2016 Horde LLC (http://www.horde.org/)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @link      http://pear.horde.org/index.php?package=ActiveSync
  * @package   ActiveSync
@@ -286,7 +286,7 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
         }
 
         if (empty($results)) {
-            $this->_logger->err(sprintf(
+            $this->_logger->warn(sprintf(
                 '[%s] Could not find state for synckey %s.',
                 $this->_procid,
                 $this->_syncKey));
@@ -558,17 +558,23 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
      *
      * @param string $devId   The device id to obtain
      * @param string $user    The user to retrieve user-specific device info for
+     * @param array  $params  Additional parameters:
+     *   - force: (boolean)  If true, reload the device info even if it's
+     *     already loaded. Used to refresh values such as device_rwstatus that
+     *     may have changed during a long running PING/SYNC. DEFAULT: false.
+     *     @since  2.31.0
      *
      * @return Horde_ActiveSync_Device  The device object
      * @throws Horde_ActiveSync_Exception
      */
-    public function loadDeviceInfo($devId, $user = null)
+    public function loadDeviceInfo($devId, $user = null, $params = array())
     {
         // See if we already have this device, for this user loaded
-        if (!empty($this->_deviceInfo) && $this->_deviceInfo->id == $devId &&
+        if (empty($params['force']) &&
+            !empty($this->_deviceInfo) &&
+            $this->_deviceInfo->id == $devId &&
             !empty($this->_deviceInfo) &&
             $user == $this->_deviceInfo->user) {
-
             return $this->_deviceInfo;
         }
 
