@@ -9,7 +9,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2015 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2016 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -57,7 +57,7 @@
  *                           combination.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- * @copyright 2010-2015 Horde LLC (http://www.horde.org/)
+ * @copyright 2010-2016 Horde LLC (http://www.horde.org/)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @link      http://pear.horde.org/index.php?package=ActiveSync
  * @package   ActiveSync
@@ -238,7 +238,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
 
         if (empty($results)) {
-            $this->_logger->err(sprintf(
+            $this->_logger->warn(sprintf(
                 '[%s] Could not find state for synckey %s.',
                 $this->_procid,
                 $this->_syncKey));
@@ -313,11 +313,6 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     public function save()
     {
-        // Update state table to remember this last synctime and key
-        $sql = 'INSERT INTO ' . $this->_syncStateTable
-            . ' (sync_key, sync_data, sync_devid, sync_mod, sync_folderid, sync_user, sync_pending, sync_timestamp)'
-            . ' VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
-
         // Prepare state and pending data
         if ($this->_type == Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC) {
             $data = (isset($this->_folder) ? serialize($this->_folder) : '');
@@ -333,30 +328,31 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         // If we are setting the first synckey iteration, do not save the
         // timestamp, otherwise we will never get the initial set of data.
         $params = array(
-            $this->_syncKey,
-            new Horde_Db_Value_Binary($data),
-            $this->_deviceInfo->id,
-            (self::getSyncKeyCounter($this->_syncKey) == 1 ? 0 : $this->_thisSyncStamp),
-            (!empty($this->_collection['id']) ? $this->_collection['id'] : Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC),
-            $this->_deviceInfo->user,
-            $pending,
-            time());
+            'sync_key' => $this->_syncKey,
+            'sync_data' => new Horde_Db_Value_Binary($data),
+            'sync_devid' => $this->_deviceInfo->id,
+            'sync_mod' => (self::getSyncKeyCounter($this->_syncKey) == 1 ? 0 : $this->_thisSyncStamp),
+            'sync_folderid' => (!empty($this->_collection['id']) ? $this->_collection['id'] : Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC),
+            'sync_user' => $this->_deviceInfo->user,
+            'sync_pending' => $pending,
+            'sync_timestamp' => time()
+        );
         $this->_logger->info(
             sprintf('[%s] Saving state: %s',
                 $this->_procid,
                 serialize(array(
-                    $params[0],
-                    $params[1],
-                    $params[2],
-                    $params[3],
-                    $params[4],
-                    $params[5],
+                    $params['sync_key'],
+                    $params['sync_data'],
+                    $params['sync_devid'],
+                    $params['sync_mod'],
+                    $params['sync_folderid'],
+                    $params['sync_user'],
                     count($this->_changes),
                     time()))
                 )
             );
         try {
-            $this->_db->insert($sql, $params);
+            $this->_db->insertBlob($this->_syncStateTable, $params);
         } catch (Horde_Db_Exception $e) {
             // Might exist already if the last sync attempt failed.
             $this->_logger->notice(
@@ -365,7 +361,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                         $this->_syncKey)
                 );
             $this->_db->delete('DELETE FROM ' . $this->_syncStateTable . ' WHERE sync_key = ?', array($this->_syncKey));
-            $this->_db->insert($sql, $params);
+            $this->_db->insertBlob($this->_syncStateTable, $params);
         }
     }
 
@@ -545,14 +541,21 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      *
      * @param string $devId   The device id to obtain
      * @param string $user    The user to retrieve user-specific device info for
+     * @param array  $params  Additional parameters:
+     *   - force: (boolean)  If true, reload the device info even if it's
+     *     already loaded. Used to refresh values such as device_rwstatus that
+     *     may have changed during a long running PING/SYNC. DEFAULT: false.
+     *     @since  2.31.0
      *
      * @return Horde_ActiveSync_Device  The device object
      * @throws Horde_ActiveSync_Exception
      */
-    public function loadDeviceInfo($devId, $user = null)
+    public function loadDeviceInfo($devId, $user = null, $params = array())
     {
         // See if we already have this device, for this user loaded
-        if (!empty($this->_deviceInfo) && $this->_deviceInfo->id == $devId &&
+        if (empty($params['force']) &&
+            !empty($this->_deviceInfo) &&
+            $this->_deviceInfo->id == $devId &&
             !empty($this->_deviceInfo) &&
             $user == $this->_deviceInfo->user) {
             return $this->_deviceInfo;
@@ -632,17 +635,22 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                 $this->_db->insert($query, $values);
             } else {
                 $this->_logger->info((sprintf(
-                    '[%s] Device entry exists for %s, updating userAgent and version.',
+                    '[%s] Device entry exists for %s, updating userAgent, version, and supported.',
                     $this->_procid,
                     $data->id)));
+                // device_supported is immutable, and only sent during the initial
+                // sync request, so only set it if it's non-empty.
                 $query = 'UPDATE ' . $this->_syncDeviceTable
-                    . ' SET device_agent = ?, device_properties = ?'
+                    . ' SET device_agent = ?, device_properties = ?' . (!empty($data->supported) ? ', device_supported = ?' : '')
                     . ' WHERE device_id = ?';
                 $values = array(
                     (!empty($data->userAgent) ? $data->userAgent : ''),
-                    serialize($data->properties),
-                    $data->id
+                    serialize($data->properties)
                 );
+                if (!empty($data->supported)) {
+                    $values[] = serialize($data->supported);
+                }
+                $values[] = $data->id;
                 $this->_db->update($query, $values);
             }
         } catch(Horde_Db_Exception $e) {
@@ -709,8 +717,10 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     public function deviceExists($devId, $user = null)
     {
         if (!empty($user)) {
-            $query = 'SELECT COUNT(*) FROM ' . $this->_syncUsersTable
-                . ' WHERE device_id = ? AND device_user = ?';
+            $query = 'SELECT COUNT(*) FROM ' . $this->_syncDeviceTable
+                . ' d INNER JOIN ' . $this->_syncUsersTable
+                . ' u on d.device_id = u.device_id'
+                . ' WHERE u.device_id = ? AND device_user = ?';
             $values = array($devId, $user);
         } else {
             $query = 'SELECT COUNT(*) FROM ' . $this->_syncDeviceTable . ' WHERE device_id = ?';
@@ -902,8 +912,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                 $results = $this->_db->selectValue($q, array($options['devId']));
                 if ($results != Horde_ActiveSync::RWSTATUS_NA &&
                     $results != Horde_ActiveSync::RWSTATUS_OK) {
-                    unset($options['user']);
-                    return $this->removeState($options);
+                    return $this->removeState(array('devId' => $options['devId']));
                 }
             } catch (Horde_Db_Exception $e) {
                 throw new Horde_ActiveSync_Exception($e);
@@ -1032,6 +1041,26 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
      }
 
+     /**
+      * Check if the UID provided was altered during the SYNC_KEY provided.
+      *
+      * @param string $uid      The UID to check.
+      * @param string $synckey  The synckey to check.
+      *
+      * @return boolean  True if the provided UID was updated during the
+      *                  SYNC for the synckey provided.
+      * @since  2.31.0
+      */
+     public  function isDuplicatePIMChange($uid, $synckey)
+     {
+        $sql = 'SELECT count(*) FROM ' . $this->_syncMapTable
+            . ' WHERE message_uid = ? AND sync_user = ? AND sync_key = ?';
+        try {
+            return $this->_db->selectValue($sql, array($uid, $this->_deviceInfo->user, $synckey));
+        } catch (Horde_Db_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
+     }
     /**
      * Return the sync cache.
      *
